using System;
using System.Reflection;
using System.Collections;
using System.Text;

public class ClassWithoutNamespace {
}

namespace DataHandling {
	public class DataSet {
		private ArrayList data = new ArrayList();

		public object this[int index] {
			get {
				return data[index];
			}

			set {
				data[index] = value;
			}
		}

		public int Count {
			get {
				return data.Count;
			}
		}

		//structure of string: [(<string>|"\i:"<integer>) {,(<string>|"\i:"<integer>)}]
		public static DataSet Parse(string dataString) {
			DataSet ds = new DataSet();
			foreach(string str in dataString.Split(',')) {
				if (str.StartsWith("\\i:")) {
					ds.Add(Int32.Parse(str.Substring(3)));
				} else {
					ds.Add(str);
				}
			}
			return ds;
		}
		
		public void Add(object o) {
			data.Add(o);
		}

		public override string ToString() {
			StringBuilder s = new StringBuilder();
			for (int i=0; i<data.Count; i++) {
				if (data[i] is Int32) {
					s.Append("\\i:");
				}
				s.Append(data[i]);
				if (i < data.Count-1) {
					s.Append(',');
				}
			}
			return s.ToString();
		}
	}

	public class DataSetUsage {
		public DataSet GetStringsInUpperCase(DataSet ds) {
			DataSet newSet = new DataSet();
			for (int i=0; i<ds.Count; i++) {
				if (ds[i] is string) {
					newSet.Add(((string)ds[i]).ToUpper());
				}
			}
			return newSet;
		}

		public int IntgerSum(DataSet ds) {
			int sum=0;
			for (int i=0; i<ds.Count; i++) {
				if (ds[i] is Int32) {
					sum += (Int32)ds[i];
				}
			}
			return sum;
		}
	}
}

namespace Calculation {
	public class IntegerCalc {
		public int SumOfTwo(int a, int b) {
			return a+b;
		}

		public int SumOfThree(int a, int b, int c) {
			return a+b+c;
		}

		public int SumOfValues(params int[] values) {
			int sum=0;
			foreach (int v in values) {
				sum += v;
			}
			return sum;
		}
	}
}


namespace InterfaceTest {
	public interface A {
		string Name {get; }
	}

	public interface B: A {
		void CallMe();
	}

	public interface C {
		int CallMeToo();
	}

	public class ITest: Calculation.IntegerCalc, B, C {
		public string Name { get { return "I am InterfaceTest.ITest"; } }
		
		public void CallMe() {
			Console.WriteLine("1+2=" + SumOfTwo(1,2).ToString());
		}
		
		public int CallMeToo() {
			Console.WriteLine("Sorry - Out of service!");
			return -1;
		}
	}
}

namespace A {

	public interface IntA {
		void CallMe();
		string Name{get;}
	}

	public interface AInterface {
	}

	namespace SpaceX {
		public class A {}
	}

	public interface ZInterface {
	}

	public interface XInterface {
	}

	public class Start : IntA, AInterface, ZInterface, XInterface {
		public enum Colors {Blue, Green, Red};

		char[] digits = {'0','1','2','3','4','5','6','7','8','9'};
		
		public delegate void Notifier (string sender);
		public delegate void SomeEvent(object source, System.EventArgs e);

		public Notifier notify;
		public event SomeEvent events;
		
		public int[] fld1;
		public int[][][] fld3;
		public char c2;
		public char c7;
		public char cc1;
		public char val;
		public int[][] fld2;

		public const int const2 = 1;
		public const byte const1 = (byte)1;

		public readonly int a = 3;

		public string Name {
			get {
				return "Start";
			}
		}

		public char this [int b, int off] {
			get {
				return digits[b+off];
			}
			set {
				digits[b+off] = value;
				//			notify("Start");
			}
		}
		
		public static void Main() {
			Console.WriteLine("I am Main in A.Start");
         
			Start s = new Start();
			s.print();
			s.print2();

			s = new C1();
			s.print();
			s.print2();
			
			((C1)s).print();
			((C1)s).print2();

			Console.WriteLine();
			B.C1 b = new B.C1();
			b.print();
			b.print2();

			Console.WriteLine();
			b = new B.C2();
			b.print();
			b.print2();

			Console.WriteLine();
			b = new B.C.C3();
			b.print();
			b.print2();

			Console.WriteLine();
			b = new B.C.Extension();
			b.print();
			b.print2();
			((B.C.Extension)b).print();
			((B.C.Extension)b).print2();
			
			
			Console.WriteLine();
			b = new B.C.Extension2();
			b.print();
			b.print2();
			((B.C.Extension)b).print();
			((B.C.Extension)b).print2();
			((B.C.Extension2)b).print();
			((B.C.Extension2)b).print2();

		}

		void print() {
			Console.WriteLine("I am A.Start");
		}

		public void EnumOutput(Colors c) {
			Console.WriteLine(c.ToString());
		}


		public virtual void print2() {
			Console.WriteLine("PRINT2 - I am A.Start");
		}
		
		public void CallMe() {
			Console.WriteLine("Start - I was called");
		}
	}

	public class C1: Start {
		// public override print(){} ist nicht mglich, da print() nicht als virtual deklariert

		public void print() {
			Console.WriteLine("I am A.C1");
		}

		public override void print2() {
			Console.WriteLine("PRINT2 - I am A.C1");
		}
	}

	public class ClassB {
		public interface C {
		}

		public interface B: C, A.IntA {
		}

		public class InnerB: B, C {
			public void CallMe() {
			}
			public string Name{
				get {
					return "A.ClassB.InnerB";
				}
			}
		}
	}

	namespace SpaceA {
		public class C {
			public C(int a) {
			}

			public void CTest() {
			}
		}

		public abstract class B {
			public abstract void M(int a, int b);

			public void T() {
			}
		}
	}
}